import moogli
import moose
from moose import neuroml
from PyQt4 import Qt, QtCore, QtGui
import sys
import os
import random
import numpy as np

app = QtGui.QApplication(sys.argv)
filename = os.path.join( os.path.split(os.path.realpath(__file__))[0]
                       , "../neuroml/PurkinjeCellPassivePulseInput/PurkinjePassive.net.xml"
                       )
moose.neuroml.loadNeuroML_L123(filename)
morphology = moogli.read_morphology_from_moose(name = "", path = "/cells[0]")
morphology.create_group( "group-1"
                       , [ "/cells[0]/BigCellCML_0[0]/Seg0_dend_1043_1044[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1033_1034[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1019_1020[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1018_1019[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1016_1017[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1539_1540[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1579_1580[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1573_1574[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1572_1573[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1569_1570[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1559_1560[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1045_1046[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1021_1022[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1020_1021[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1581_1582[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1580_1581[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1046_1047[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1022_1023[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1027_1028[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1023_1024[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1028_1029[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1025_1026[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1024_1025[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1026_1027[0]"
                         ]
                       , 10.0
                       , 200.0
                       , [1.0, 0.0, 0.0, 1.0]
                       , [0.0, 1.0, 0.0, 1.0]
                       )
morphology.create_group( "group-2"
                       , [ "/cells[0]/BigCellCML_0[0]/Seg0_dend_1076_1077[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1072_1073[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1099_1100[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1096_1097[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1108_1109[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1135_1136[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_35_36[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_655_656[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_535_536[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_716_717[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_14_15[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_87_88[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_236_237[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_218_219[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_172_173[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_152_153[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_363_364[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_362_363[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_344_345[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_341_342[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_320_321[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_312_313[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_453_454[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_449_450[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_437_438[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_426_427[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_418_419[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_409_410[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_407_408[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_396_397[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_394_395[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_390_391[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_389_390[0]"
                         , "/cells[0]/BigCellCML_0[0]/Seg0_dend_1079_1080[0]"
                         ]
                       , 0.0
                       , 1.0
                       , [0.0, 1.0, 0.0, 1.0]
                       , [0.0, 0.0, 1.0, 1.0]
                       )
def callback(morphology, viewer):
    morphology.set_color( "group-1"
                        , np.random.random_sample((24,)) * (100.0 - 20.0) + 20.0
                        )
    morphology.set_color( "group-2"
                        , np.random.random_sample((34,))
                        )
    viewer.roll(0.05, 0)
    viewer.pitch(0.05, 1)
    viewer.yaw(0.05, 2)
    return True

viewer = moogli.DynamicMorphologyViewerWidget(morphology)
viewer.showMaximized()
viewer.split_horizontally(0)
viewer.split_vertically(1)
viewer.zoom(0.5, 0)
viewer.zoom(0.5, 1)
viewer.zoom(0.5, 2)
viewer.set_callback(callback)
app.exec_()



"""
This script demonstrates how to use moogli to carry out a simulation and
simultaneously update the visualizer. Also another viewer shows the compartment
rm values. The visualizers remain active while the simulation is running.
Once the simulation finishes, a graph comes up, showing the soma vm values.
"""

import moogli
import moose
from moose import neuroml
from PyQt4 import Qt, QtCore, QtGui
import sys
import os
import random
import numpy as np
import math
from PyQt4.QtCore import *
from PyQt4.QtGui import *
from matplotlib.backends.backend_qt4agg import FigureCanvasQTAgg as FigureCanvas
from matplotlib.figure import Figure

# The QApplication class manages the GUI application's
# control flow and main settings
app = QtGui.QApplication(sys.argv)

# Load model from the neuroml file into moose
filename = os.path.join( os.path.split(os.path.realpath(__file__))[0]
                       , "../neuroml/PurkinjeCellPassivePulseInput/PurkinjePassive.net.xml"
                       )
popdict, projdict = moose.neuroml.loadNeuroML_L123(filename)

# setting up hsolve object for each neuron
for popinfo in popdict.values():
    for cell in popinfo[1].values():
        solver = moose.HSolve(cell.path + "/hsolve")
        solver.target = cell.path

# create a table to store vm values
table = moose.Table("/soma_table")
moose.connect( table
             , 'requestOut'
             , moose.element("/cells[0]/BigCellCML_0[0]/Seg0_soma_0[0]")
             , "getVm"
             )

# reinit moose to bring to a reliable initial state.
moose.reinit()

SIMULATION_DELTA = 0.001
SIMULATION_TIME  = 0.03

ALL_COMPARTMENTS = map( lambda x : x.path
                      , moose.wildcardFind("/cells[0]/##[ISA=CompartmentBase]")
                      )

BASE_RM_VALUE = min( map( lambda x : moose.element(x).Rm
                        , ALL_COMPARTMENTS
                        )
                   )
PEAK_RM_VALUE = max( map( lambda x : moose.element(x).Rm
                        , ALL_COMPARTMENTS
                        )
                   )
BASE_RM_COLOR = [0.0, 1.0, 0.0, 0.1]
PEAK_RM_COLOR = [1.0, 0.0, 0.0, 1.0]

BASE_VM_VALUE = -0.065
PEAK_VM_VALUE = -0.060
BASE_VM_COLOR = [1.0, 0.0, 0.0, 0.1]
PEAK_VM_COLOR = [0.0, 0.0, 1.0, 1.0]

"""Create a class for plotting vm"""
class PlotWidget(FigureCanvas):
    """Ultimately, this is a QWidget (as well as a FigureCanvasAgg, etc.)."""
    def __init__(self, parent=None, width=5, height=4, dpi=100):
        fig = Figure(figsize=(width, height), dpi=dpi)
        self.axes = fig.add_subplot(111)
        # We want the axes cleared every time plot() is called
        self.axes.hold(False)
        FigureCanvas.__init__(self, fig)
        self.setParent(parent)

        FigureCanvas.setSizePolicy(self,
                                   QtGui.QSizePolicy.Expanding,
                                   QtGui.QSizePolicy.Expanding)
        FigureCanvas.updateGeometry(self)

def create_vm_visualizer():
    # Moogli requires a morphology object. Create a morphology object
    # by reading the geometry details from all objects of type CompartmentBase
    # inside /cells[0]
    vm_morphology = moogli.read_morphology_from_moose(name = "", path = "/cells[0]")


    # Create a named group of compartments called 'group-all'
    # which will contain all the compartments of the model.
    # Each group has a strict upper and lower limit for the
    # variable which is being visualized.
    # Both limits map to colors provided to the api.
    # The value of the variable is linearly mapped to a color value
    # lying between the upper and lower color values.

    vm_morphology.create_group( "group-all"    # group name
                              , ALL_COMPARTMENTS  # sequence of compartments belonging to this group
                              , BASE_VM_VALUE   # base value of variable
                              , PEAK_VM_VALUE   # peak value of variable
                              , BASE_VM_COLOR # color corresponding to base value
                              , PEAK_VM_COLOR # color corresponding to peak value
                           )


    # set initial color of all compartments in accordance with their vm
    vm_morphology.set_color( "group-all"
                           , map( lambda x : moose.element(x).Vm
                                , ALL_COMPARTMENTS
                                )
                           )

    # instantiate the visualizer with the morphology object created earlier
    vm_viewer = moogli.DynamicMorphologyViewerWidget(vm_morphology)

    # Callback function will be called by the visualizer at regular intervals.
    # The callback can modify both the morphology and viewer object's properties
    # since they are passed as arguments.
    def callback(morphology, viewer):
        # run simulation for 1 ms
        moose.start(SIMULATION_DELTA)

        # change color of all the compartments according to their vm values.
        # a value higher than peak value will be clamped to peak value
        # a value lower than base value will be clamped to base value.
        morphology.set_color( "group-all"
                            , map( lambda x : moose.element(x).Vm
                                 , ALL_COMPARTMENTS
                                 )
                            )
        # if the callback returns true, it will be called again.
        # if it returns false it will not be called again.
        # the condition below ensures that simulation runs for 1 sec
        if moose.element("/clock").currentTime < SIMULATION_TIME : return True
        else:
            plot_widget = PlotWidget()
            xs = np.linspace(0, moose.element("/clock").runTime, len(table.vector))
            ys = table.vector.copy()
            plot_widget.axes.plot(xs, ys)
            plot_widget.draw()
            plot_widget.show()
            plot_widget.setWindowTitle("Soma Vm")
            return False


    # set the callback function to be called after every idletime milliseconds
    vm_viewer.set_callback(callback, idletime = 0)

    # make sure that entire model is visible
    vm_viewer.pitch(math.pi / 2)
    vm_viewer.zoom(0.25)
    vm_viewer.setWindowTitle("Vm Visualization")
    return vm_viewer

def create_rm_visualizer():
    # Moogli requires a morphology object. Create a morphology object
    # by reading the geometry details from all objects of type CompartmentBase
    # inside /cells[0]
    morphology = moogli.read_morphology_from_moose(name = "", path = "/cells[0]")

    # Create a named group of compartments called 'group-all'
    # which will contain all the compartments of the model.
    # Each group has a strict upper and lower limit for the
    # variable which is being visualized.
    # Both limits map to colors provided to the api.
    # The value of the variable is linearly mapped to a color value
    # lying between the upper and lower color values.
    morphology.create_group( "group-all"    # group name
                           , ALL_COMPARTMENTS  # sequence of compartments belonging to this group
                           , BASE_RM_VALUE # base value of variable
                           , PEAK_RM_VALUE # peak value of variable
                           , BASE_RM_COLOR # color corresponding to base value
                           , PEAK_RM_COLOR # color corresponding to peak value
                           )

    # set initial color of all compartments in accordance with their rm
    morphology.set_color( "group-all"
                        , map( lambda x : moose.element(x).Rm
                             , ALL_COMPARTMENTS
                             )
                        )

    # instantiate the visualizer with the morphology object created earlier
    rm_viewer = moogli.MorphologyViewerWidget(morphology)

    # make sure that entire model is visible
    rm_viewer.pitch(math.pi / 2)
    rm_viewer.zoom(0.25)
    rm_viewer.setWindowTitle("Rm")
    return rm_viewer

vm_visualizer = create_vm_visualizer()
rm_visualizer = create_rm_visualizer()
vm_visualizer.show()
rm_visualizer.show()

# http://stackoverflow.com/questions/15861839/error-upon-app-shutdown-qglcontextmakecurrent-cannot-make-invalid-context-cu
def delete_gl_widget():
    global vm_visualizer
    global rm_visualizer
    vm_visualizer.setParent(None)
    del vm_visualizer
    rm_visualizer.setParent(None)
    del rm_visualizer

QApplication.instance().aboutToQuit.connect( delete_gl_widget )


# Enter the main event loop and wait until exit() is called.
# It is necessary to call this function to start event handling.
# The main event loop receives events from the window system and
# dispatches these to the application widgets.
app.exec_()



import moogli
import moose
from moose import neuroml
from PyQt4 import Qt, QtCore, QtGui
import sys
import os

app = QtGui.QApplication(sys.argv)
filename = os.path.join( os.path.split(os.path.realpath(__file__))[0]
                       , "/PurkinjeCellPassivePulseInput/PurkinjePassive.net.xml"
                       )

moose.neuroml.loadNeuroML_L123(filename)
network_distal = moogli.simulator.moose.read_network( name = ""
                                                    , path = "/cells[0]"
                                                    , radius = moogli.DISTAL
                                                    )
viewer_distal = moogli.NetworkViewer(network_distal)
viewer_distal.setWindowTitle("Distal Radius")

network_averaged = moogli.simulator.moose.read_network( name = ""
                                                      , path = "/cells[0]"
                                                      , radius = moogli.AVERAGED
                                                      )
viewer_averaged = moogli.NetworkViewer(network_averaged)
viewer_averaged.setWindowTitle("Averaged Radius")

network_proximal_distal = moogli.simulator.moose.read_network( name = ""
                                                             , path = "/cells[0]"
                                                             , radius = moogli.PROXIMAL_DISTAL
                                                             )
viewer_proximal_distal = moogli.NetworkViewer(network_proximal_distal)
viewer_proximal_distal.setWindowTitle("Continuous Variation in Radius")

viewer_distal.show()
viewer_averaged.show()
viewer_proximal_distal.show()
app.exec_()



from PyQt4 import QtGui
from PyQt4 import QtCore
from PyQt4 import Qt
from PyQt4 import QtOpenGL


# from PyQt4.QtGui import *
from _moogli import *

class DynamicNetworkViewer(NetworkViewer):
    _timer = QtCore.QTimer()

    def set_callback(self,callback, idletime = 0):
        self.callback = callback
        self.idletime = idletime
        self._timer.timeout.connect(self.start_cycle)
        self.start_cycle()

    def start_cycle(self):
        if self.isVisible():
            if self.callback(self.get_morphology(), self):
                self._timer.start(self.idletime)
            else:
                self._timer.timeout.disconnect(self.start_cycle)
            self.update()
        else:
            self._timer.start(self.idletime)


# __all__ = [ "Morphology"
#           , "MorphologyViewer"
#           , "MorphologyViewerWidget"
#           , "DynamicMorphologyViewerWidget"
#           ]



import moose

DISTAL          = 0
AVERAGED        = 1
PROXIMAL_DISTAL = 2

def read_network(name = "", path = "", radius = DISTAL):
    network = _moogli.Network(name, 1)
    compartments = moose.wildcardFind(path + "/##[ISA=CompartmentBase]")
    neuron_ids = set( map( lambda compartment : compartment.parent.path
                         , compartments
                         )
                    )
    for neuron_id in neuron_ids:
        neuron = moogli.Neuron(neuron_id)
        moose_compartments = moose.wildcardFind(neuron_id + "/####[ISA=CompartmentBase]")
        for moose_compartment in moose_compartments:
            compartment = moogli.Compartment(moose_compartment.path)
            distal_diameter = moose_compartment.diameter
            try:
                parent_compartment = moose_compartment.neighbors["raxial"][0]
                proximal_diameter  = parent_compartment.diameter
            except IndexError:
                proximal_diameter = distal_diameter

            if   radius == DISTAL          :
                proximal_diameter = distal_diameter
            elif radius == AVERAGED        :
                distal_diameter = proximal_diameter =  ( distal_diameter
                                                       + proximal_diameter
                                                       ) / 2.0
            compartment.add_geometry( [ moose_compartment.x * 10000000
                                      , moose_compartment.y * 10000000
                                      , moose_compartment.z * 10000000
                                      , distal_diameter     * 10000000
                                      ]
                                    , [ moose_compartment.x0 * 10000000
                                      , moose_compartment.y0 * 10000000
                                      , moose_compartment.z0 * 10000000
                                      , proximal_diameter    * 10000000
                                      ]
                                    )
            neuron.add_compartment(compartment)
        network.add_neuron(neuron)
    return network



# -*- coding: utf-8 -*-

"""
The following resources have been really helpful in making this setup script -

https://github.com/numpy/numpy/blob/master/setup.py
https://packaging.python.org/en/latest/index.html
https://github.com/pypa/sampleproject/blob/master/setup.py
https://docs.python.org/2/distutils/apiref.html
http://stackoverflow.com/questions/3207219/how-to-list-all-files-of-a-directory-in-python
"""



from setuptools import setup, Extension
import sipdistutils
import sys
import os

remaining = sys.argv[1:]
sys.argv = [ sys.argv[0]
           , 'build_ext'
           , '--sip-opts=-I/usr/share/sip/PyQt4/ -e -g -x VendorID -t WS_X11 -x PyQt_NoPrintRangeBug -t Qt_4_8_0 -x Py_v3 -g'
           ]
sys.argv.extend(remaining)

here = os.path.dirname(__file__)

# print(sys.argv)
# RESOURCES -
# https://docs.python.org/2.7/distutils/apiref.html?highlight=setup#distutils.core.setup
# http://pyqt.sourceforge.net/Docs/sip4/distutils.html#ref-distutils
# https://wiki.python.org/moin/EmbedingPyQtTutorial


# list of object files to be passed to the linker.
# These files must not have extensions, as the default extension for the compiler is used.
os.environ["CC"]="g++"
extra_objects           =   [
                            ]

# list of libraries to link against
libraries               =   [ "QtCore"
                            , "QtGui"
                            , "QtOpenGL"
                            , "osg"
                            , "osgFX"
                            , "osgUtil"
                            , "osgFX"
                            , "osgGA"
                            , "osgQt"
                            , "osgAnimation"
                            , "osgViewer"
                            , "osgQt"
                            , "osgManipulator"
                            , "osgText"
                            ]

# list of directories to search for libraries at link-time
library_dirs            =   [
                            ]

# # list of directories to search for shared (dynamically loaded) libraries at run-time
runtime_library_dirs    =   [ ]

# additional command line options for the compiler command line
extra_compile_args      =   [ "-O3"
                            , "-std=c++0x"
                            , "-Wno-reorder"
                            , "-Wno-overloaded-virtual"
                            ]

# additional command line options for the linker command line
extra_link_args         =   [ "-fPIC"
                            , "-shared"
                            ]

#specify include directories to search
include_dirs            =   [ "."
                            , "/usr/share/sip/PyQt4/"
                            , "/usr/include/qt4/"
                            , "/usr/include/qt4/QtCore/"
                            , "/usr/include/qt4/QtGui/"
                            , "/usr/include/qt4/QtOpenGL/"
                            , "/usr/share/sip/PyQt4/QtCore/"
                            , "/usr/share/sip/PyQt4/QtGui/"
                            , "/usr/share/sip/PyQt4/QtOpenGL/"
                            , "./moogli/include/"
                            ]

# define pre-processor macros
define_macros           =   [
                            ]

# undefine pre-processor macros
undef_macros            =   [
                            ]

moogli = Extension( name                  =   "_moogli"
                  , sources               =   [ "moogli/src/core/Network.cpp"
                                              , "moogli/src/core/Neuron.cpp"
                                              , "moogli/src/core/Compartment.cpp"
                                              , "moogli/src/core/Voxel.cpp"
                                              , "moogli/src/view/NetworkViewer.cpp"
                                              , "moogli/src/mesh/CylinderMesh.cpp"
                                              , "moogli/src/mesh/SphereMesh.cpp"
                                              , "moogli/src/utility/record.cpp"
                                              , "moogli/src/utility/globals.cpp"
                                              , "moogli/src/utility/conversions.cpp"
                                              , "moogli/moc/NetworkViewer.moc.cpp"
                                              ]
                  , include_dirs          =   include_dirs
                  , extra_compile_args    =   extra_compile_args
                  , extra_link_args       =   extra_link_args
                  , library_dirs          =   library_dirs
                  , libraries             =   libraries
                  , extra_objects         =   extra_objects
                  , runtime_library_dirs  =   runtime_library_dirs
                  , define_macros         =   define_macros
                  , undef_macros          =   undef_macros
                  )

with open('requirements.txt') as f:
    required = f.read().splitlines()

long_description = open("README.rst").read()

scripts_dir = os.path.join(here, "scripts")
scripts = [os.path.join(scripts_dir,fn) for fn in next(os.walk(scripts_dir))[2]]

setup( name             =   'moogli'
     , author           =   'Aviral Goel'
     , author_email     =   'aviralg@ncbs.res.in'
     , maintainer       =   'Aviral Goel'
     , maintainer_email =   'aviralg@ncbs.res.in'
     , version          =   "0.1.0"
     , url              =   ''
     , download_url     =   ''
     , description      =   "A 3D visualizer for neuronal networks"
     , long_description =   long_description
     , classifiers      =   [ 'Development Status :: 3 - Alpha'
                            , 'Environment :: X11 Applications :: Qt'
                            , 'Intended Audience :: Science/Research'
                            , "License :: OSI Approved :: GNU General Public License v2 (GPLv2)"
                            , 'Programming Language :: Python :: 2.6'
                            , 'Programming Language :: Python :: 2.7'
                            , 'Programming Language :: C++'
                            , 'Natural Language :: English'
                            , 'Operating System :: OS Independent'
                            , 'Topic :: Scientific/Engineering'
                            ]
     , license          =   'GPLv2'
     , requires         =   required
     , packages         =   [ "moogli"
                            , "moogli.reader"
                            , "moogli.simulator"
                            ]
     , ext_modules      =   [ moogli ]
     , cmdclass         =   { 'build_ext': sipdistutils.build_ext
                            }
     , scripts          =   scripts
)



